package ru.pricepulse.parsingservice.ozon_parser.service.parsing;

import java.util.ArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicBoolean;

import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;
import ru.pricepulse.parsingservice.enumeration.Category;
import ru.pricepulse.parsingservice.ozon_parser.service.MarketplaceParsingService;

@Slf4j
@Service
@Profile("ozon")
public class OzonParsingService implements MarketplaceParsingService {

    private final AtomicBoolean stopFlag = new AtomicBoolean(false);

    private final ExecutorService pageExecutorService = Executors.newFixedThreadPool(12);

    private final OzonCategoryPageParsingService categoryPageParsingService;

    public OzonParsingService(OzonCategoryPageParsingService categoryPageParsingService) {
        this.categoryPageParsingService = categoryPageParsingService;
    }

    public void processCategory(String url) {
        int pageIndex = 1;

        while (!stopFlag.get()) {
            int finalPageIndex = pageIndex;
            String pageUrl = url + "&page=" + finalPageIndex;
            pageExecutorService.submit(() -> categoryPageParsingService.parseCategoryPage(pageUrl, Category.LAPTOP, stopFlag));
            ++pageIndex;
        }
        if (stopFlag.get()) {
            pageExecutorService.shutdownNow();
        }
    }

}