package ru.pricepulse.parsingservice.service;

import java.util.ArrayList;
import java.util.List;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.pricepulse.parsingservice.ozon_parser.service.dto.ParsedData;
import ru.pricepulse.parsingservice.persistence.entity.PriceHistoryEntity;
import ru.pricepulse.parsingservice.persistence.entity.PriceHistoryId;
import ru.pricepulse.parsingservice.persistence.entity.ProductEntity;
import ru.pricepulse.parsingservice.persistence.repository.ProductPriceRepository;
import ru.pricepulse.parsingservice.persistence.repository.ProductRepository;

@Slf4j
@Service
@RequiredArgsConstructor
public class ProductService {

    private final ProductRepository productRepository;

    private final ProductPriceRepository productPriceRepository;

    @Transactional
    public void saveBatch(List<ParsedData> parsedData) {
        var products = new ArrayList<ProductEntity>();
        var prices = new ArrayList<PriceHistoryEntity>();
        parsedData.forEach(product -> processParsedProduct(product, prices, products));
        productRepository.saveAll(products);
        log.info("Сохранили пачку товаров {}", products.size());
        productPriceRepository.saveAll(prices);
        log.info("Сохранили историю цен {}", prices.size());
    }

    private void processParsedProduct(ParsedData product,
                                      ArrayList<PriceHistoryEntity> prices,
                                      ArrayList<ProductEntity> products) {
        var priceHistoryEntity = getPriceHistory(product);
        prices.add(priceHistoryEntity);
        if (productRepository.existsByUrl(product.getUrl())) {
            log.debug("Запись {} уже есть", product.getUrl());
            return;
        }
        var productEntity = getProduct(product);
        products.add(productEntity);
    }

    private PriceHistoryEntity getPriceHistory(ParsedData product) {
        var priceHistoryId = new PriceHistoryId();
        priceHistoryId.setProductUrl(product.getUrl());
        var priceHistory = new PriceHistoryEntity();
        priceHistory.setId(priceHistoryId);
        priceHistory.setPrice(product.getPrice());
        return priceHistory;
    }

    private ProductEntity getProduct(ParsedData product) {
        var productEntity = new ProductEntity();
        productEntity.setCategory(product.getCategory());
        productEntity.setBrand(product.getBrand());
        productEntity.setProductName(product.getProductName());
        productEntity.setUrl(product.getUrl());
        productEntity.setMarketplace(product.getMarketplace());
        productEntity.setImageUrl(product.getImageUrl());
        return productEntity;
    }
}
